import { Entitlement, User } from "cdp-js/dist/api/iam/types";
import { FunctionComponent, ComponentChild } from 'preact';
export interface SidebarNavigationItem {
    /** Identifier for the sidebar item */
    name: string;
    /** Name shown in UI */
    displayName: string;
    /**
     * Preact icon component
     * See @cloudera/cuix-core/icons/preact for available all CUIX icons in preact format for easy use.
     * If you want to pass in custom icon and/or SVG, you need to wrap it in a Preact component. See
     * `getIconComponentForSvg` in utils.tsx for an example.
     *
     * We could also have gone with nodes instead of components e.g.
     * icon: <MyIcon /> vs IconComponent: MyIcon
     * which often seems better. However, that would introduce some headache since we'd then need
     * to include preact as well as the jsx pragma header on that page, compared to just importing
     * an icon and adding it.
     */
    IconComponent: FunctionComponent;
    /** When provided an <a> will be rendered with given url */
    url?: string;
    /** When provided a <button> will be rendered with the given handler */
    handler?: (name: string, event: Event) => any;
    type: 'navigation';
    /** When set to true, we display an additonal exclamation mark next to the menu item */
    showExclamationMark?: boolean;
}
export declare type SidebarAccordionSubItem = Omit<SidebarNavigationItem, 'IconComponent'>;
export interface SidebarSpacerItem {
    type: 'spacer';
}
export interface SidebarAccordionItem {
    type: 'accordion';
    name: string;
    displayName: string;
    IconComponent: FunctionComponent;
    children: SidebarAccordionSubItem[];
}
export interface SidebarSectionItem {
    type: 'section';
    name: string;
    displayName: string;
    children: (SidebarNavigationItem | SidebarSpacerItem | SidebarAccordionItem)[];
}
export declare type SidebarItem = SidebarNavigationItem | SidebarSpacerItem | SidebarAccordionItem | SidebarSectionItem;
export interface HelpItem {
    IconComponent?: FunctionComponent;
    href?: string;
    baseUrl?: string;
    path?: string;
    target?: string;
    /** Internal name, might be used for tests and similar */
    name: string;
    /** Name shown to users */
    displayName: string;
    /** An optional entitlement to prevent the item from being displayed */
    entitlement?: string;
}
export interface BuildDetails {
    descriptionList: {
        term: ComponentChild;
        description: ComponentChild;
    }[];
    docs?: HelpItem[];
}
/**
 * Optional object that contains either a url or a handler that leads to a new what's new page
 */
export interface WhatsNewDetails {
    /** When provided this url leads to a What's new page from a button in the sidebar footer. */
    url?: string;
    handler?: () => any;
}
/**
 * Certain things we can have the sidebar fetch on-demand, and/or be optional, and/or provide
 * ourselves if we have already fetched such data e.g. entitlements.
 */
export declare type InternalOrExternalFetchedData<T> = T | 'auto' | 'loading' | null;
export interface SidebarProps {
    /** Used on images on alt|title attributes */
    productDisplayName: string;
    /** Which app in the app switcher to highlight as active. Use name from apps.v3.json */
    productName: string | null;
    /** Determines which item in the sidebar is marked as active - should be the current page */
    activeItemName?: string;
    /** List of items to server as the main navigation for the app */
    sidebarItems: SidebarItem[];
    /** List of items displayed under "helpful links" when you open up the "Help" menu */
    helpItems?: HelpItem[];
    /** The url to be used for the icon logo in the header */
    productIconLogoSrc: string;
    /** The url to be used for the text logo in the header (e.g. showing Cloudera + Product Name) */
    productTextLogoSrc: string;
    /** Url we navigate to if you click the logo in the header */
    logoHref: string;
    /** If you need to hide the app switcher from the header */
    hideAppSwitcher?: boolean;
    /** Hide the link to the user profile in the user drawer that shows if you click the user item in the menu */
    hideUserItemProfileLink?: boolean;
    /** Hide the help menu in the nav footer */
    hideHelpMenu?: boolean;
    /** Hide the tour menu in the nav footer */
    hideTourMenu?: boolean;
    /** URL to hit when user clicks the logout link in the user popover. */
    logoutUrl?: string;
    /**
     * When provided a logout <button> will be rendered with the given handler.
     * Useful for applications running outside the control plane that need to handle logging out
     * with their own http library.
     */
    logoutHandler?: () => any;
    /** Color for the product. Used as highlight color and background color for the collapsed header. */
    productColor: string;
    /** Topic name for the contextual help panel. */
    helpTopic?: string;
    /**
     * Specific location of help topics in case a customer's environment requires it.
     * The default is '/commonconsole/downloads/help' inside the control plane.
     * If baseHelpUrl is provided, we assume you are providing a url relative to your
     * application, or an absolute url.
     * Using baseHelpUrl will not be relative to the common console in CDP.
     */
    baseHelpUrl?: string;
    /**
     * Entitlements can be automatically fetched via 'auto' or
     * manually set via array of entitlements and optionally 'loading' to indicate loading
     * Set to null if you don't want to use entitlements
     */
    entitlements: InternalOrExternalFetchedData<Entitlement[]>;
    /**
     * User can be automatically fetched via 'auto' or
     * manually set via array of entitlements and optionally 'loading' to indicate loading
     * Set to null if you don't want to use entitlements
     */
    user: InternalOrExternalFetchedData<User>;
    /**
     * Should the sidebar by default be collapsed.
     * If you can, avoid specifying this so our experience is consistent across the board.
     * */
    defaultCollapsed?: boolean;
    /**
     * Short version number displayed at the bottom of the sidebar. If you don't have one yet
     * use an empty string.
     */
    versionNumber: string;
    /**
     * Optional build details. When specified clicking the version number, will open up a drawer
     * showing the build details.
     */
    buildDetails?: BuildDetails;
    /**
     * Optional object that contains either a url or a handler that leads to a new what's new page
     */
    whatsNew?: WhatsNewDetails;
    /**
     * Optional "tag" that can be displayed along with the product text logo at
     * the top of the sidebar. We've used this to display a temporary "BETA" tag,
     * for example.
     */
    productDisplayNameTag?: string;
    /**
     * Class applied to the optional "tag". This prop is exposed so that products
     * can style the tag as needed.
     */
    productDisplayNameTagClass?: string;
}
