export declare enum FeatureFlagTypes {
    BOOL = "BOOL",
    STRING = "STRING",
    SELECT = "SELECT"
}
export interface FeatureFlagBase {
    id: string;
    name: string;
    description?: string;
    defaultValue: any;
    type: FeatureFlagTypes;
    value?: any;
    hidden?: boolean;
}
export interface FeatureFlagSelect extends FeatureFlagBase {
    values: {
        name: string;
        value: string;
    }[];
    type: FeatureFlagTypes.SELECT;
}
export declare type FeatureFlag = FeatureFlagBase | FeatureFlagSelect;
declare type FeatureFlags = FeatureFlag[];
interface FeatureFlagsManagerConfig {
    flags: FeatureFlags;
    consoleName: string;
}
/**
 * Feature flags
 *
 * This is a generic feature flag component that can be adopted by consoles if they want
 * UI only feature flags for development. This is useful when we are working on bigger features
 * that still are in-progress, but live in master and can allow you to iterate better with other
 * dev team members.
 *
 * Feature flags are stored in local storage, so are per-user browser specific.
 *
 * To adopt this for your console. You'll need to create a `featureFlagsManager.ts` inside the
 * console, import `FeatureFlagsManager` and export an instance of it. See example in sigma console.
 *
 * To adopt the UI for modifying feature flags include the `FeatureFlagsView` in a console page.
 *
 * Once that is setup, import you instance of the featureFlagManager and you can do something like:
 *
 *  ```
 *  import featureFlagsManager, {
 *    MY_FEATURE_FLAG
 *  } from 'your/console/path/featureFlagsManager';
 *  //.....
 *  if (featureFlagsManager.getValue(MY_FEATURE_FLAG) ....
 *  ```
 *
 * to check if feature flag is enabled or not (or get it's value).
 */
export declare class FeatureFlagsManager {
    flags: FeatureFlags;
    flagsById: Map<string, FeatureFlag>;
    consoleName: string;
    constructor(config: FeatureFlagsManagerConfig);
    /**
     * Internal method to generate local storage key
     */
    private getStorageKeyForFlagId;
    /**
     * Internal method to fetch the values from the store upon start.
     */
    private getInitalFeatureFlagValue;
    /**
     * Call this method from user facing code to determining if flag is set or not.
     */
    getValue(flagConfig: FeatureFlag): any;
    /**
     * Call to set a feature flag. Note that this should not be utilized by customer facing code.
     * This method will throw if there local storage problems. But that is acceptable as it
     * should only be called from dev pages.
     */
    setValue(flagId: string, newValue: any): void;
    getAllFeatureFlags(): FeatureFlags;
}
export {};
