// Copyright (c) 2018 Cloudera, Inc. All rights reserved.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
export var FeatureFlagTypes;
(function (FeatureFlagTypes) {
    FeatureFlagTypes["BOOL"] = "BOOL";
    FeatureFlagTypes["STRING"] = "STRING";
    FeatureFlagTypes["SELECT"] = "SELECT";
})(FeatureFlagTypes || (FeatureFlagTypes = {}));
/**
 * Feature flags
 *
 * This is a generic feature flag component that can be adopted by consoles if they want
 * UI only feature flags for development. This is useful when we are working on bigger features
 * that still are in-progress, but live in master and can allow you to iterate better with other
 * dev team members.
 *
 * Feature flags are stored in local storage, so are per-user browser specific.
 *
 * To adopt this for your console. You'll need to create a `featureFlagsManager.ts` inside the
 * console, import `FeatureFlagsManager` and export an instance of it. See example in sigma console.
 *
 * To adopt the UI for modifying feature flags include the `FeatureFlagsView` in a console page.
 *
 * Once that is setup, import you instance of the featureFlagManager and you can do something like:
 *
 *  ```
 *  import featureFlagsManager, {
 *    MY_FEATURE_FLAG
 *  } from 'your/console/path/featureFlagsManager';
 *  //.....
 *  if (featureFlagsManager.getValue(MY_FEATURE_FLAG) ....
 *  ```
 *
 * to check if feature flag is enabled or not (or get it's value).
 */
var FeatureFlagsManager = /** @class */ (function () {
    function FeatureFlagsManager(config) {
        var _this = this;
        this.consoleName = config.consoleName;
        this.flags = config.flags.map(function (flag) { return (__assign(__assign({}, flag), { value: _this.getInitalFeatureFlagValue(flag) })); });
        this.flagsById = new Map();
        this.flags.forEach(function (flag) {
            _this.flagsById.set(flag.id, flag);
        });
    }
    /**
     * Internal method to generate local storage key
     */
    FeatureFlagsManager.prototype.getStorageKeyForFlagId = function (flagId) {
        return "cdp.feature-flag.".concat(this.consoleName, ".").concat(flagId);
    };
    /**
     * Internal method to fetch the values from the store upon start.
     */
    FeatureFlagsManager.prototype.getInitalFeatureFlagValue = function (flag) {
        var value;
        try {
            value = localStorage.getItem(this.getStorageKeyForFlagId(flag.id));
            value = JSON.parse(value);
        }
        catch (err) {
            value = flag.defaultValue;
        }
        if (value === null) {
            value = flag.defaultValue;
        }
        return value;
    };
    /**
     * Call this method from user facing code to determining if flag is set or not.
     */
    FeatureFlagsManager.prototype.getValue = function (flagConfig) {
        var flag = this.flagsById.get(flagConfig.id);
        if (!flag) {
            console.warn("Trying to get value for unknown feature flag ".concat(flagConfig.id));
            return false;
        }
        return flag.value;
    };
    /**
     * Call to set a feature flag. Note that this should not be utilized by customer facing code.
     * This method will throw if there local storage problems. But that is acceptable as it
     * should only be called from dev pages.
     */
    FeatureFlagsManager.prototype.setValue = function (flagId, newValue) {
        var flag = this.flagsById.get(flagId);
        if (!flag) {
            throw new Error("Unknown flag ".concat(flagId));
        }
        var key = this.getStorageKeyForFlagId(flag.id);
        if (newValue === flag.defaultValue) {
            localStorage.removeItem(key);
        }
        else {
            localStorage.setItem(key, JSON.stringify(newValue));
        }
        flag.value = newValue;
    };
    FeatureFlagsManager.prototype.getAllFeatureFlags = function () {
        return this.flags;
    };
    return FeatureFlagsManager;
}());
export { FeatureFlagsManager };
