import { ReactNode } from 'react';
/**
 * Defines a "facet" -- a property of the data to be filtered that can be
 * filtered against.
 */
export interface Facet<T> {
    /**
     * The name of the facet that will be displayed. It is also used as an ID for
     * the facet by the `Filter` framework, so make it unique.
     *  */
    label: string;
    /**
     * An optional name to be used in filter output instead of the label.
     */
    name?: string;
    /**
     * The initially selected values for the facet.
     */
    initialValues?: T[];
}
export type FacetItem = string | {
    /**
     * The value that should be output for this item.
     */
    value: string;
    /**
     * The `ReactNode` that will be rendered in the dropdown.
     */
    display?: ReactNode;
    /**
     * The string that will be shown in the dropdown and in the pill when this
     * item is selected. If `display` is provided, it will be used in the
     * dropdown instead.
     */
    label: string;
    /**
     * Must be unique. Required if `value` is not unique.
     */
    key?: string;
    /**
     * The item will be disabled if `true`. The primary use for this is to prevent deselecting default values in a multiple value facet. This is ignored by `SingleValueFilter`.
     */
    disabled?: boolean;
};
export interface BasicFacet extends Facet<FacetItem> {
    /**
     * The items to display in the dropdown menu.
     */
    items: FacetItem[];
    /**
     * If true, a search box will be displayed.
     */
    search?: boolean;
    /**
     * If truthy, multiple-select mode is enabled. To show the batch
     * operation and apply/cancel controls, pass an object with `apply`
     * set to `true`.
     */
    multiple?: boolean | {
        apply?: boolean;
    };
}
export interface CustomFacet<T = any> extends Facet<T> {
    /**
     * A function that renders a custom dropdown.
     */
    dropdownRender: FacetDropdownRender;
    /**
     * A function that can be used to get a display string from a value of type T.
     * @default String()
     */
    toDisplayValue?: (value: T) => string;
}
/**
 * Represents the current state of a facet. Used internally.
 */
export interface FacetState<T = any> {
    label: string;
    values: T[];
    show?: boolean;
    isFixed?: boolean;
    open?: boolean;
    facet?: BasicFacet | CustomFacet<T>;
}
export type FilterState = FacetState[];
type FacetOutput<T = unknown> = T[];
export interface FilterOutput {
    [key: string]: FacetOutput;
}
export type FacetChangeHandler = (facetState: FacetState) => void;
export interface FacetDropdownProps {
    label: string;
    name?: string;
    values?: FacetItem[];
    onClose?: () => void;
    onChange?: FacetChangeHandler;
    style?: React.CSSProperties;
    open?: boolean;
}
export interface FilterProps extends Omit<FacetDropdownProps, 'style' | 'open'> {
    items: FacetItem[];
}
export type FacetDropdownRender = (onChange: FacetChangeHandler, close: () => void) => React.ReactNode;
/**
 * A base type to use when building a custom filter component. All custom filter
 * components should be passed these props via `dropdownRender`.
 */
export interface CustomFilterProps {
    /**
     * A custom filter component must call this to provide filter value changes
     * to the framework.
     */
    onChange?: FacetChangeHandler;
    /**
     * Call this to imperatively close the dropdown when needed.
     */
    close?: () => void;
}
export {};
