import React from 'react';
import { ApiError } from "cuix/dist/types/types";
import type { SpinSize } from 'antd/es/spin';
export interface WithStateHandlingProps {
    error?: ApiError;
    infoError?: ApiError;
    fetching?: boolean;
    fetchStatus?: string;
    isEmpty?: boolean;
    lastFetched?: number;
    useInlineLoader?: boolean;
    refreshing?: boolean;
}
export interface Config {
    getEmptyState?: (props: any) => any;
    getNotFoundState?: (props: any) => any;
    getErrorState?: (props: any) => any;
    getFetchingState?: (props: any) => React.ReactNode;
    loadingIndicatorSize?: SpinSize;
}
/**
 * HOC for handling common states for components. Handles following states:
 *
 * ERROR: If error prop is defined it is considered a fatal error and an error component is
 * rendered. UI for NOT_FOUND errors can be customized with the getNotFoundState configuration.
 *
 * FETCHING: If fetching prop is true the Loading component is rendered. Fetching state is determined
 * by either the boolean fetching flag or the fetchStatus value. See apiBusReducer.FetchStatus for
 * more info.
 *
 * EMPTY: If isEmpty prop is true a custom empty state component is rendered. There is no default UI
 * for empty state so getEmptyState configuration option is required if isEmpty is provided.
 *
 * All these props are optional so you can opt out of handling a specific state while still handling
 * other by not providing the corresponding prop.
 *
 * @param {object} config Configuration object
 * @param {props => ReactNode} config.getEmptyState Function that returns a ReactNode to render when
 * component is in empty state
 * @param {props => ReactNode} config.getNotFoundState Function that returns a ReactNode to render
 * when component has a NOT_FOUND error
 */
declare const withStateHandling: (config?: Config) => <P extends WithStateHandlingProps>(WrappedComponent: React.ComponentType<P>) => {
    new (props: (P & WithStateHandlingProps) | Readonly<P & WithStateHandlingProps>): {
        isFetching: () => boolean | NonNullable<(P & WithStateHandlingProps)["fetching"]>;
        render(): any;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<P & WithStateHandlingProps>) => {} | Pick<{}, K> | null) | Pick<{}, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<P & WithStateHandlingProps> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<P & WithStateHandlingProps>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<P & WithStateHandlingProps>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<P & WithStateHandlingProps>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<P & WithStateHandlingProps>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): void;
    };
    new (props: P & WithStateHandlingProps, context: any): {
        isFetching: () => boolean | NonNullable<(P & WithStateHandlingProps)["fetching"]>;
        render(): any;
        context: any;
        setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<P & WithStateHandlingProps>) => {} | Pick<{}, K> | null) | Pick<{}, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<P & WithStateHandlingProps> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<{}>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<P & WithStateHandlingProps>, prevState: Readonly<{}>): any;
        componentDidUpdate?(prevProps: Readonly<P & WithStateHandlingProps>, prevState: Readonly<{}>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<P & WithStateHandlingProps>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<P & WithStateHandlingProps>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<P & WithStateHandlingProps>, nextState: Readonly<{}>, nextContext: any): void;
    };
    WrappedComponent: React.ComponentType<P>;
    contextType?: React.Context<any> | undefined;
};
export default withStateHandling;
