// Copyright (c) 2020 Cloudera, Inc. All rights reserved.
import { useSelector } from 'react-redux';
import { selectors, Status } from "cuix/dist/reducers/apiBusReducer";
/* ******** ApiBus Hooks ************
 *
 * Convenience hooks for api bus since they are likely to see a lot of usage.
 *
 * Since the linting for hooks is based on name e.g. `use*` for hooks we currently
 * keep their name verbose and exported individually.
 *
 * e.g.
 *    useApiBusItem()  <--- triggers hook linting/checks
 *    apiBusHooks.useItem() <--- does *not* trigger hook linting/checks
 *
 * Hence for now we'll want to keep them verbosely named and exported independently
 */
/**
 * Convenience hook to get api bus item. See underlying reducer for more description.
 *
 * ```tsx
 * const MyComponent = () => {
 *   const item = useApiBusItem<SomeItemType>('channelName');
 *   return ...;
 * }
 * ```
 */
export var useApiBusItem = function (channel, itemPath) {
    return useSelector(function (state) { return selectors.getItem(state, channel, itemPath); });
};
/**
 * Convenience hook to get api bus items. See underlying reducer for more description
 */
export var useApiBusItems = function (channel) {
    return useSelector(function (state) { return selectors.getItems(state, channel); });
};
/**
 * Convenience hook to get api bus error. See underlying reducer for more description
 */
export var useApiBusError = function (channels) {
    return useSelector(function (state) { return selectors.getError(state, channels); });
};
/**
 * Convenience hook to get api bus status. See underlying reducer for more description
 */
export var useApiBusFetchingStatus = function (channels) {
    return useSelector(function (state) { return selectors.getStatus(state, channels); });
};
/**
 * Convenience hook to get is fetching status. See underlying reducer for more description
 */
export var useApiBusIsFetching = function (channels) {
    return [Status.FETCHING_INITIAL, Status.FETCHING_FOLLOWING].includes(useApiBusFetchingStatus(channels));
};
/**
 * Convenience hook to get details on an api bus item. Details include the bus item, whether it is fetching, and possible bus error.
 * See the underlying reducer for more description
 */
export var useApiBusItemDetails = function (channel, itemPath) {
    return {
        item: useApiBusItem(channel, itemPath),
        isFetching: useApiBusIsFetching(channel),
        error: useApiBusError(channel)
    };
};
/**
 * Convenience hook to get details on api bus items. Details include the bus items, whether it is fetching, and possible bus error.
 * See the underlying reducer for more description
 */
export var useApiBusItemsDetails = function (channel) {
    return {
        items: useApiBusItems(channel),
        isFetching: useApiBusIsFetching(channel),
        error: useApiBusError(channel)
    };
};
