import { ApiError } from "cuix/dist/types/types";
/**
 * Type predicate to narrow the response from useApiData, useSubmissionApi etc.
 *
 * The return value from these hooks contains { data, fetching, error } among other things.
 * When using this type predicate it will guarantee that the request has
 * returned data and is not loading and has no error.
 *
 * This means you can access that data property `request.data` and TS will not
 * type it as optional anymore.
 *
 * Example:
 *
 * ```tsx
 * const req = useApiData(...);
 *
 * // data is optional, you'll have to use the ? operator or other check for it existence e.g.
 * req.data?.property
 *
 * if (!isSuccessfullyCompleted(req)) {
 *   return <LoadingAndErrorHandler request={req} />;
 * }
 *
 * // data guaranteed to be set at this point, and TS will
 * // type it as such so you can access it directly.
 * req.data.property
 * ```
 */
export declare function isSuccessfullyCompleted<T>(req: {
    data?: T;
    error?: ApiError;
    fetching: boolean;
}): req is {
    data: T;
    error: undefined;
    fetching: false;
};
/**
 * @deprecated We had a typo. Use `isSuccessfullyCompleted` instead. Leaving this to not be a breaking change. This export will likely be removed in a future version.
 */
export declare const isSucessfullyCompleted: typeof isSuccessfullyCompleted;
/**
 * In some cases where you work with the ReturnType of `useApiData` derived hook
 * as an actual type, you might handle loading in a upper component, but then
 * pass the "request" directly down into a child component. In that case, then
 * you know that the data is fetched, so you'll want to narrow the type to avoid
 * redoing loading handlers etc in the child component.
 *
 * This is a **rare** use-case as in most cases you'll pass down the data object
 * itself instead of the "request" object. Where it can be be useful is
 * if you attach additional info to the request object, but even then often you
 * can pass additional data down as separate props.
 *
 * Do note that a downside of this approach currently, is that the request
 * object is not stable itself as it is generated new on every render though
 * some individual properties on it such as `data` are stable, so `useMemo` or
 * similar won't work on the "request" object itself.
 *
 * Example:
 *
 * ```tsx
 * const useEnv = () => {
 *   const req = useApiData({ api: fetchEnv, fetchParams: {}});
 *   return {
 *     ...req,
 *    additionalEnvData,
 *    otherUsefulContext
 *   }
 * }
 * type EnvRequest = ReturnType<typeof useEnv>;
 *
 * const Parent = () => {
 *   const request = useEnv();
 *   if (!isSuccessfullyCompleted(req)) {
 *     return <LoadingAndErrorHandler request={req} />;
 *   }
 *   // at this point request.data is always defined, otherwise
 *   // we returned above.
 *   return <Child req={request} />;
 * }
 *
 * const Child: React.FC<{req: RequestWithResolvedData<EnvRequest>}> = ({ req }) => {
 *   // Now req.data is always defined so you don't need to
 *   // If you pass in a "request" that might have undefined data in the Parent
 *   // TS will now raise an error.
 *   return <>{req.data.environment.name} and {req.otherUsefulContext.envOwner}</>;
 * }
 * ```
 *
 */
export declare type RequestWithResolvedData<T extends {
    data: any;
}> = T extends {
    data?: infer X;
} ? T & {
    data: X;
} : never;
