// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
/**
 * Type predicate to narrow the response from useApiData, useSubmissionApi etc.
 *
 * The return value from these hooks contains { data, fetching, error } among other things.
 * When using this type predicate it will guarantee that the request has
 * returned data and is not loading and has no error.
 *
 * This means you can access that data property `request.data` and TS will not
 * type it as optional anymore.
 *
 * Example:
 *
 * ```tsx
 * const req = useApiData(...);
 *
 * // data is optional, you'll have to use the ? operator or other check for it existence e.g.
 * req.data?.property
 *
 * if (!isSuccessfullyCompleted(req)) {
 *   return <LoadingAndErrorHandler request={req} />;
 * }
 *
 * // data guaranteed to be set at this point, and TS will
 * // type it as such so you can access it directly.
 * req.data.property
 * ```
 */
export function isSuccessfullyCompleted(req) {
    return req.data !== undefined && !req.fetching && !req.error;
}
/**
 * @deprecated We had a typo. Use `isSuccessfullyCompleted` instead. Leaving this to not be a breaking change. This export will likely be removed in a future version.
 */
export var isSucessfullyCompleted = isSuccessfullyCompleted;
