import { HooksConfig } from './useInternalApiData';
/**
 * Hook for retrieving API data. Fetches immediately (if it can't reuse existing data)
 *
 * Example:
 *
 * ```js
 * const { data, error, fetching } = useApiData({
 *   api: wa.getOperationExecutionDetails,
 *   fetchParams: {
 *     id: 'xyz',
 *   }
 * });
 *
 * if (error) { ... }
 * if (fetching) { ... }
 *
 * // or use the helper
 * if (!isSuccessfullyCompleted(req)) {
 *   return <LoadingAndErrorHandler request={req} />;
 * }
 * return <div>{data.operationName}</div>;
 *
 * ```
 *
 * If we do not want to make the request right away, we can pass in NO_FETCH to the `fetchParams`.
 * Though if your primary goal is submitting data as opposed to reading data look at the
 * `useSubmissionApi` hook instead.
 *
 * ```js
 * const { data, error, fetching } = useApiData({
 *   api: shouldFetch ? wa.getOperationExecutionDetails : null,
 *   fetchParams: shouldFetch ? {
 *     id: 'xyz',
 *   } : NO_FETCH
 * });
 * ```
 *
 * In some cases we want to use "composite" APIs, typically when we make requests
 * that depend on each other.
 *
 * ```js
 * const compositeApi = makeCompositeApi(async ({ id, otherParam }) => {
 *   const firstResp = await myApi.firstCall({ id });
 *   const secondResp = await myApi.otherApi({
 *     name: firstResp.obj.name,
 *     otherParam
 *   });
 *   return secondResp;
 * });
 * //...
 * const req = useApiData({
 *   api: compositeApi,
 *   fetchParams: { id: 'xyz', otherParam: true }
 * });
 * //...
 * ```
 *
 */
export default function useApiData<ApiParams, ResponseType>(apiConfig: HooksConfig<ResponseType, ApiParams>): {
    fetching: boolean;
    fetchingInitial: boolean;
    data: ResponseType | undefined;
    listData: any[];
    error: import("../../types/types").ApiError | undefined;
    lastFetched: number | undefined;
    refresh: () => void;
};
