// Copyright (c) 2020 Cloudera, Inc. All rights reserved.
import useInternalApiData, { NO_FETCH } from './useInternalApiData';
import { useEffect } from 'react';
/**
 * Hook for retrieving API data. Fetches immediately (if it can't reuse existing data)
 *
 * Example:
 *
 * ```js
 * const { data, error, fetching } = useApiData({
 *   api: wa.getOperationExecutionDetails,
 *   fetchParams: {
 *     id: 'xyz',
 *   }
 * });
 *
 * if (error) { ... }
 * if (fetching) { ... }
 *
 * // or use the helper
 * if (!isSuccessfullyCompleted(req)) {
 *   return <LoadingAndErrorHandler request={req} />;
 * }
 * return <div>{data.operationName}</div>;
 *
 * ```
 *
 * If we do not want to make the request right away, we can pass in NO_FETCH to the `fetchParams`.
 * Though if your primary goal is submitting data as opposed to reading data look at the
 * `useSubmissionApi` hook instead.
 *
 * ```js
 * const { data, error, fetching } = useApiData({
 *   api: shouldFetch ? wa.getOperationExecutionDetails : null,
 *   fetchParams: shouldFetch ? {
 *     id: 'xyz',
 *   } : NO_FETCH
 * });
 * ```
 *
 * In some cases we want to use "composite" APIs, typically when we make requests
 * that depend on each other.
 *
 * ```js
 * const compositeApi = makeCompositeApi(async ({ id, otherParam }) => {
 *   const firstResp = await myApi.firstCall({ id });
 *   const secondResp = await myApi.otherApi({
 *     name: firstResp.obj.name,
 *     otherParam
 *   });
 *   return secondResp;
 * });
 * //...
 * const req = useApiData({
 *   api: compositeApi,
 *   fetchParams: { id: 'xyz', otherParam: true }
 * });
 * //...
 * ```
 *
 */
export default function useApiData(apiConfig) {
    var _a = useInternalApiData(apiConfig), fetching = _a.fetching, fetchingInitial = _a.fetchingInitial, unstarted = _a.unstarted, data = _a.data, listData = _a.listData, error = _a.error, lastFetched = _a.lastFetched, startFetchIfNeeded = _a.startFetchIfNeeded;
    useEffect(function () {
        startFetchIfNeeded();
    });
    return {
        // we're fetching right away, so even if the api bus channel isn't marked as fetching but unstarted
        // we'll consider that fetching. Otherwise we'd have a potential "blip" at the start where the
        // it's marked as not fetching even though we're in the process of starting a fetch.
        fetching: apiConfig.fetchParams === NO_FETCH ? fetching : fetching || unstarted,
        fetchingInitial: apiConfig.fetchParams === NO_FETCH ? fetchingInitial : fetchingInitial || unstarted,
        data: data,
        listData: listData,
        error: error,
        lastFetched: lastFetched,
        refresh: function () { return startFetchIfNeeded({ force: true }); }
    };
}
