import { RequestMethod } from "cdp-js/dist/lib/apiCommon";
import { RelaxedRequestMethod } from "cdp-js/dist/lib/compositeApi";
/** Pass this to `fetchParams` to stop fetching. */
export declare const NO_FETCH: unique symbol;
export interface HooksConfig<ResponseType, ApiParams> {
    /**
     * The API we use. This can be a regular API (most common) such as
     * `wa.getOperationExecutionConfiguration`
     * or it can be for example a compositeApi (see `makeCompositeApi` in `cdp-js`)
     */
    api: RequestMethod<ApiParams, ResponseType> | RelaxedRequestMethod<ApiParams, ResponseType>;
    /**
     * Params to send along with api call.
     *
     * Set to `NO_FETCH` to disable fetching. While initially it might feel
     * unintuitive that we disable through the `fetchParam` parameter, consider
     * that in many cases the reason we are disable fetching is because we don't
     * have the required data to fetch, and it follows that if we don't have the
     * required data, then we can't build a complete `fetchParams` object.
     *
     * Example:
     * ```
     * fetchParams: clusterId ? { clusterId } : NO_FETCH;
     * ```
     */
    fetchParams: ApiParams | typeof NO_FETCH;
    /**
     * function to determine if the request should be refreshed
     * e.g. for example if asking for the status of something
     * and the status is running, we might want to refresh until
     * the status is settled
     */
    shouldRefresh?: (data: ResponseType) => boolean;
    /**
     * Initial delay for refreshing. Will backoff exponentially until
     * until the request is changed.
     */
    initialRefreshDelay?: number;
    /**
     * Optionally specify the channel name to use instead of generating one automatically
     */
    channelName?: string;
    /**
     * Tells underlying apiBus not to clear the cache on inter-page navigation
     */
    preserve?: boolean;
    /**
     * Use a list request instead (see apiBus list api).
     * **NOTE**: this does not work well with TS typing so the returning
     * data is on a new property `any[]`.
     * TODO: [future] investigate more how to improve this typing. (arne)
     */
    listRequest?: boolean;
    /**
     * List of APIs that will invalidate this API call and cause it to be refreshed/re-ran.
     * This is useful if you have calls that modify the state e.g. "updateXYZ".
     * For example if you have a `listXYZ` call, you might want to use `invalidateOn` to be
     * `[deleteXYZ, updateXYZ]`, which would cause `listXYZ` to be refreshed/called if either
     * of `deleteXYZ` or `updateXYZ` where called successfully.
     */
    invalidateOn?: RequestMethod<any, any>[];
}
/**
 * The underlying hook use by useApiData and useOnDemandApiData which fetches API data.
 *
 * It utilizes apiBus for fetching, and generates channels names based on the api and fetch params.
 * This means that the request will be cached and shared with any other callers.
 *
 * This is currently not meant to be used directly, but through useApiData or useOnDemandApiData.
 */
declare function useInternalApiData<ApiParams, ResponseType>(apiConfig: HooksConfig<ResponseType, ApiParams>): {
    fetching: boolean;
    fetchingInitial: boolean;
    unstarted: boolean;
    data: ResponseType | undefined;
    listData: any[];
    error: import("../../types/types").ApiError | undefined;
    lastFetched: number | undefined;
    startFetchIfNeeded: (config?: {
        force?: boolean;
        onlyIfInvalid?: boolean;
    }) => void;
};
export default useInternalApiData;
