// Copyright (c) 2020 Cloudera, Inc. All rights reserved.
import { useEffect, useRef } from 'react';
import { useSelector, useDispatch, useStore } from 'react-redux';
import { selectors as apiSelectors, actions as apiActions } from "cuix/dist/reducers/apiBusReducer";
import { getDelayHelper } from "cuix/dist/utils/delayGenerator";
import { generateChannelName } from './misc';
/** A "dummy" channel name that will point to an empty channel */
var EMPTY_CHANNEL_NAME = '_useApiData_empty_channel_';
/** Pass this to `fetchParams` to stop fetching. */
export var NO_FETCH = Symbol('NO_FETCH');
/**
 * The underlying hook use by useApiData and useOnDemandApiData which fetches API data.
 *
 * It utilizes apiBus for fetching, and generates channels names based on the api and fetch params.
 * This means that the request will be cached and shared with any other callers.
 *
 * This is currently not meant to be used directly, but through useApiData or useOnDemandApiData.
 */
function useInternalApiData(apiConfig) {
    var _a;
    var fetchParams = apiConfig.fetchParams;
    var store = useStore();
    var channelName = apiConfig.channelName ||
        (fetchParams === NO_FETCH
            ? EMPTY_CHANNEL_NAME
            : generateChannelName({ api: apiConfig.api, fetchParams: fetchParams }));
    var data = useSelector(function (state) {
        return apiSelectors.getItem(state, channelName);
    });
    var error = useSelector(function (state) { return apiSelectors.getError(state, channelName); });
    var fetching = useSelector(function (state) { return apiSelectors.isFetching(state, channelName); });
    var unstarted = useSelector(function (state) { return apiSelectors.isUnstarted(state, channelName); });
    var isInvalidated = useSelector(function (state) {
        return apiSelectors.getIsInvalidated(state, channelName);
    });
    var dispatch = useDispatch();
    // this could be optimized using useCallback or such, but in order to do that we'd
    // have to do a bunch of deep comparisons. Might be more worthwhile to focus on that
    // if we see perf problems.
    var startFetchIfNeeded = function (config) {
        config = config || {};
        if (fetchParams === NO_FETCH) {
            return;
        }
        // we'll want to verify against the latest store state to make sure the request should be made.
        // We make this call from `useEffect` hooks which by the time they are run, might be acting
        // on slightly stale data (e.g. the data that was fetched when the main hook body ran).
        var state = store.getState();
        if (!config.force &&
            !config.onlyIfInvalid &&
            apiSelectors.isFetchingOrCompleted(state, channelName)) {
            return;
        }
        if (config.onlyIfInvalid && !apiSelectors.getIsInvalidated(state, channelName)) {
            return;
        }
        if (apiConfig.listRequest) {
            dispatch(
            // With a TS upgrade we need to tighten the `fetchParam` type to be compatible.
            // In practice this is not a problem, so loosening the type for now. Besides
            // ApiBus itself is very simply typed.
            apiActions.submitListRequest(apiConfig.api, fetchParams, channelName, {
                preserve: apiConfig.preserve
            }));
        }
        else {
            dispatch(apiActions.submitRequest(apiConfig.api, fetchParams, channelName, {
                preserve: apiConfig.preserve,
                invalidateOn: apiConfig.invalidateOn
            }));
        }
    };
    if (((_a = apiConfig.invalidateOn) === null || _a === void 0 ? void 0 : _a.length) && apiConfig.listRequest) {
        // This combination is not tested, so will throw an error
        // for now instead of causing hard-to-debug issues/frustration.
        // It's also possible that it works out of the box, but don't want to assume.
        throw new Error('Having both .listRequest and .invalidateOn specified is not yet tested, so will throw for now');
    }
    var shouldRefresh = apiConfig.shouldRefresh;
    // Delay generator needs to persist as a mutable value
    // since it keeps track of last delay and increases them
    var delayGenerator = useRef(getDelayHelper());
    var lastFetchedCurrentChannel = useSelector(function (state) {
        return apiSelectors.getLastFetched(state, channelName);
    });
    var lastFetched = useRef(lastFetchedCurrentChannel);
    if (lastFetchedCurrentChannel) {
        lastFetched.current = lastFetchedCurrentChannel;
    }
    useEffect(function () {
        // start a new fetch if our existing fetch is marked as invalidated
        if (isInvalidated) {
            startFetchIfNeeded({ onlyIfInvalid: true });
        }
    });
    useEffect(function () {
        // when we get a new request (or new starting delay) we'll want to
        // reset our our delay generator since the delays are shorter in
        // the beginning
        delayGenerator.current = getDelayHelper(apiConfig.initialRefreshDelay);
    }, [channelName, apiConfig.initialRefreshDelay]);
    // responsible for refreshing (if enabled)
    // will be torn down and redone on each render
    // hence why we need to use timeSinceLastFetch
    useEffect(function () {
        if (!lastFetchedCurrentChannel || fetching || !shouldRefresh || !data || !shouldRefresh(data)) {
            return;
        }
        var timeSinceLastFetch = Date.now() - lastFetchedCurrentChannel;
        var delayLeft = Math.max(delayGenerator.current.getDelay() - timeSinceLastFetch, 0);
        var timeoutId = setTimeout(function () {
            delayGenerator.current.next();
            // can't use hooks (e.g. useSelector) inside an effect
            // so we get the state directly.
            var state = store.getState();
            var isFetching = apiSelectors.isFetching(state, channelName);
            if (!isFetching) {
                // if not fetching we start one, otherwise this will run this effect again
                // when the data being currently fetched is complete since the existing
                // fetch is identical with this (same channelName).
                startFetchIfNeeded({ force: true });
            }
        }, delayLeft);
        return function () {
            if (timeoutId) {
                clearTimeout(timeoutId);
            }
        };
    });
    var outputData;
    if (!apiConfig.listRequest) {
        outputData = data;
    }
    var outputDataList = [];
    if (apiConfig.listRequest && data) {
        var items = data.items;
        outputDataList = items || [];
    }
    return {
        fetching: fetching,
        fetchingInitial: fetching && !lastFetched.current,
        unstarted: unstarted,
        data: outputData,
        listData: outputDataList,
        error: error,
        lastFetched: lastFetched.current,
        startFetchIfNeeded: startFetchIfNeeded
    };
}
export default useInternalApiData;
