var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
import { useState, useMemo, useCallback, useEffect, useLayoutEffect } from 'react';
import debounce from 'lodash/debounce';
/**
 * Debounced a setState value/setter.
 * This could be used in a child component if you don't want to update the parent component for
 * example while typing, because that would be expensive and cause UI delays.
 *
 * Usage:
 *
 * ```js
 * // Child input debouncing within component
 * const ChildFilter = ({ value, setValue }) => {
 *   const [internalValue, setInternalValue] = useDebouncedProxyState(value, setValue);
 *   return <Input value={internalValue} onChange(e => setInternalValue(e.target.value)) />;
 * }
 *
 * // search param debouncing
 * const SearchFilter = () => {
 *   const [searchTerm, setSearchTerm] = useSearchParamState('searchTerm', '');
 *   const [internalSearchTerm, setInternalSearchTerm] = useDebouncedProxyState(
 *     searchTerm,
 *     setSearchTerm
 *   );
 *   return <Input value={internalSearchTerm} onChange(e => setInternalSearchTerm(e.target.value)) />;
 * }
 * ```
 */
export var useDebouncedProxyState = function (externalValue, setExternalValue, debounceDelayMs) {
    if (debounceDelayMs === void 0) { debounceDelayMs = 200; }
    var _a = __read(useState(externalValue), 2), value = _a[0], setValue = _a[1];
    var debouncedCallback = useMemo(function () {
        return debounce(setExternalValue, debounceDelayMs);
    }, [setExternalValue, debounceDelayMs]);
    var set = useCallback(function (val) {
        setValue(val);
        debouncedCallback(val);
    }, [debouncedCallback]);
    useEffect(function () {
        setValue(externalValue);
    }, [externalValue]);
    useLayoutEffect(function () {
        return function () {
            // We don't want the component to change any state *after* it has
            // been unmounted, as this could lead to errors/bugs etc. While
            // this bug is rare since our default delay is small, it can be
            // observed for example in fast e2e UI tests where we type into
            // a filter field and then click a link. If the link was already
            // present before filtering, we might click the link before the
            // debounced input was changed and hence executing the `setExternalValue`
            // on a different page (which could for example change the URL search params)
            debouncedCallback.cancel();
        };
    }, [debouncedCallback]);
    return [value, set];
};
export default useDebouncedProxyState;
