// Copyright (c) 2022 Cloudera, Inc. All rights reserved.
import { useCallback } from 'react';
import { actions as preferenceActions, getPreference } from "cuix/dist/reducers/preferenceReducer";
import { useDispatch, useSelector } from 'react-redux';
/**
 * Hook to interact with our preference state, which is backed by localStorage, which will persist
 * between sessions. The `storageKey` is the key suffix used by localStorage to store the value.
 *
 * Works similarly to `React.useState`.
 *
 * ```tsx
 * // sets up a state stored under the 'sigma.showNotice' key and with
 * // default value of `true`.
 * const [value, setValue] = usePreferenceState('sigma.showNotice', true)
 *
 * // setting direct value
 * setValue(false);
 *
 * // setting via fn
 * setValue(oldState => !oldState);
 * ```
 *
 * See `preferenceReducer` and `storageSaga` for more details.
 */
export var usePreferenceState = function (storageKey, defaultValue) {
    var value = useSelector(function (state) {
        return getPreference(state, storageKey, defaultValue);
    });
    var dispatch = useDispatch();
    var setValue = useCallback(function (valueOrFn) {
        var newValue = valueOrFn instanceof Function ? valueOrFn(value) : valueOrFn;
        if (newValue !== value) {
            dispatch(preferenceActions.savePreference(storageKey, newValue));
        }
    }, [dispatch, storageKey, value]);
    return [value, setValue];
};
