// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
import { useMemo, useRef } from 'react';
/**
 * Hook to filter an array be search term, only when search term or array has changed.
 *
 * If the search term contains spaces, it will be broken up in to parts and, each part
 * match individually.
 *
 * **Note**: Altering `accessorFn` will not cause re-renders.
 */
var useSearchFiltering = function (
/** List of items we want to filter */
allItems, 
/** search term */
searchTerm, 
/** function to return the searchable text or text[] given an item */
accessorFn) {
    // We'll use a ref here as we dont' want it to cause re-computation if it changes
    // e.g. if you specify an inline function when calling this hook, the function will
    // technically change on each render (since it is redeclared - even if the function
    // does the same thing)
    var accessorFnRef = useRef(accessorFn);
    accessorFnRef.current = accessorFn;
    var filteredItems = useMemo(function () {
        if (!allItems) {
            return;
        }
        if (!searchTerm) {
            return allItems;
        }
        var terms = searchTerm.split(' ').map(function (term) { return term.toLowerCase(); });
        var isMatch = function (text) {
            if (!text) {
                return false;
            }
            var lowercaseText = text.toLowerCase();
            return terms.every(function (term) { return lowercaseText.includes(term); });
        };
        return allItems.filter(function (item) {
            var text = accessorFnRef.current(item);
            if (!Array.isArray(text)) {
                return isMatch(text);
            }
            return text.some(isMatch);
        });
    }, [allItems, searchTerm]);
    return filteredItems;
};
export default useSearchFiltering;
