import { ApiError } from "cuix/dist/types/types";
import { RequestMethod } from "cdp-js/dist/lib/apiCommon";
declare type UseSubmissionApiCallback<ResponseType> = (data?: ResponseType, error?: ApiError) => any;
/**
 * Hook for using with data API submission.
 *
 * It's a convenience method for making update/delete etc requests based on user actions.
 *
 * It takes up to two argument, first which is the required API call and optionally a callback
 * that will be called once back after the API call has been made (e.g. for event based actions).
 *
 * They payload sent to the API is sent with the fetch call.
 *
 * ### Example 1
 * ```js
 *   const [{ data, error, fetching }, submitRequest] = useSubmissionApi(wa.submitXYZFormData);
 *   if (data) {
 *     return <>Success</>
 *   }
 *   return <Form onSubmit={data => submitRequest(data)} error={error} fetching={fetching} ..../>
 * ```
 *
 * ### Example 2
 * If you want to do something special on success (such as navigate to another page)
 * you can provide a callback (so it only gets called once per success/error)
 *
 * ```js
 * const [{ data, error, fetching }, submitRequest] =
 *     useSubmissionApi(wa.submitXYZFormData, (data, err) => {
 *        if (!err) {
 *           redirectTo('wxm')
 *        }
 *     });
 * return <Form onSubmit={data => submitRequest(data)} ..../>
 * ```
 *
 *   (this could also be accomplished in your code with a useMemo e.g
 *     `useMemo(() => data && redirectTo('wxm')), [data])`
 *   )
 *
 */
export default function useSubmissionApi<ApiParams extends object, ResponseType>(api: RequestMethod<ApiParams, ResponseType>, onComplete?: UseSubmissionApiCallback<ResponseType>): readonly [{
    data: ResponseType | undefined;
    error: ApiError | undefined;
    fetching: boolean;
}, (fetchParams: ApiParams) => boolean];
export {};
