// Copyright (c) 2020 Cloudera, Inc. All rights reserved.
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { useRef, useCallback, useState, useEffect } from 'react';
import { useApiBusItemDetails } from '../apiBusHooks';
import { useDispatch } from 'react-redux';
import { actions as apiActions } from "cuix/dist/reducers/apiBusReducer";
/** used to generate unique channel names */
var idCounter = 0;
/**
 * Hook for using with data API submission.
 *
 * It's a convenience method for making update/delete etc requests based on user actions.
 *
 * It takes up to two argument, first which is the required API call and optionally a callback
 * that will be called once back after the API call has been made (e.g. for event based actions).
 *
 * They payload sent to the API is sent with the fetch call.
 *
 * ### Example 1
 * ```js
 *   const [{ data, error, fetching }, submitRequest] = useSubmissionApi(wa.submitXYZFormData);
 *   if (data) {
 *     return <>Success</>
 *   }
 *   return <Form onSubmit={data => submitRequest(data)} error={error} fetching={fetching} ..../>
 * ```
 *
 * ### Example 2
 * If you want to do something special on success (such as navigate to another page)
 * you can provide a callback (so it only gets called once per success/error)
 *
 * ```js
 * const [{ data, error, fetching }, submitRequest] =
 *     useSubmissionApi(wa.submitXYZFormData, (data, err) => {
 *        if (!err) {
 *           redirectTo('wxm')
 *        }
 *     });
 * return <Form onSubmit={data => submitRequest(data)} ..../>
 * ```
 *
 *   (this could also be accomplished in your code with a useMemo e.g
 *     `useMemo(() => data && redirectTo('wxm')), [data])`
 *   )
 *
 */
export default function useSubmissionApi(api, onComplete) {
    // we'll use a initial dummy name here to point to a channel that will never exists.
    var _a = __read(useState('useSubmission-channel-name-that-does-not-exists'), 2), channelId = _a[0], setChannelId = _a[1];
    var dispatch = useDispatch();
    var _b = useApiBusItemDetails(channelId), item = _b.item, error = _b.error, isFetching = _b.isFetching;
    var previousState = useRef({
        data: undefined,
        error: undefined,
        fetching: false
    });
    // we'll use the same format as useApiData hooks
    // we'll also hang on to the existing data and/or error while a
    // new request is made because for example in a form setting we want to show
    // the existing error until a new reuqest is finished not started.
    var state = {
        data: item || previousState.current.data,
        error: item ? undefined : error || previousState.current.error,
        fetching: isFetching
    };
    var fetch = useCallback(function (fetchParams) {
        if (state.fetching) {
            console.warn('Existing request in progress. Ignoring new one.');
            return false;
        }
        var newChannelId = "useSubmission-".concat(api.methodName, "-").concat(idCounter);
        idCounter += 1;
        dispatch(apiActions.submitRequest(api, fetchParams, newChannelId));
        setChannelId(newChannelId);
        return true;
    }, [state.fetching, dispatch, api]);
    // we'll only fire the onComplete callback, once the req is done, but since we don't have
    // direct access to that event, we'll fire once we notice the data or the error has been changed.
    var fireCompleteCallback = previousState.current.data !== state.data || previousState.current.error !== state.error;
    // since the callback can be used for side-effects, it should not be triggered
    // directly in the rendering cycle (which would cause react to complain)
    useEffect(function () {
        if (fireCompleteCallback && onComplete) {
            onComplete(state.data, state.error);
        }
    });
    previousState.current = state;
    return [state, fetch];
}
