import type { ModalFuncProps } from 'antd/es/modal';
import { RootState } from "cuix/dist/types/react-types";
import { ApiError } from "cuix/dist/types/types";
import { AnyAction } from 'redux';
export declare const types: {
    readonly SUBMIT_REQUEST: "SUBMIT_REQUEST";
    readonly REQUEST_SUCCESS: "REQUEST_SUCCESS";
    readonly REQUEST_FAILURE: "REQUEST_FAILURE";
    readonly PREPARE_CHANNEL: "PREPARE_CHANNEL";
    readonly REQUEST_FLUSH: "REQUEST_FLUSH";
    readonly IS_CONFIRMED: "IS_CONFIRMED";
};
export declare type ActionType = typeof types[keyof typeof types];
export declare type Channels = string | string[];
interface ChannelMessageConfig {
    /** i18n key for confirm  message */
    confirmKey: string;
    /** i18n confirm message template data */
    confirmParams: string[];
    /** confirm modal params from https://ant.design/components/modal/#API */
    confirmModalParams?: ModalFuncProps;
    /** i18n key for notification message */
    notifyKey: string;
    /** i18n notification message template data */
    notifyParams: string[];
}
declare type UserProvidedMessageConfig = string | Partial<ChannelMessageConfig> | string[];
interface ChannelArgs {
    /** tells a list action to fetch all records */
    listAll?: boolean;
    /** When true, apiBusSaga will show a confirmation modal with a message before submitting the request */
    confirm?: boolean;
    /** When true, apiBusSaga will show a notification toaster on the top-right portion of the page when the API request is finished. */
    notify?: boolean;
    /** Configuration for text on confirmation and notification dialogs. */
    messageConfig?: ChannelMessageConfig | UserProvidedMessageConfig;
    numItemsToFetch?: number;
    currentItems?: any[];
    currentNextToken?: number | string;
    /**
     * Skip flushing (e.g. on navigate) unless flushed directly by channel name
     */
    preserve?: boolean;
    /**
     * Given a list of APIs it will mark the channel as `invalidated: true` if
     * any of the given APIs are successfully finished. Poentially re-fetching of
     * the resource is left up to you.
     */
    invalidateOn?: {
        methodName?: string;
        serviceName?: string;
    }[];
}
declare type UserProvidedChannelArgs = Pick<ChannelArgs, Exclude<keyof ChannelArgs, 'messageConfig'>> & {
    messageConfig?: UserProvidedMessageConfig;
    [key: string]: unknown;
};
interface RequestParams {
    [key: string]: any;
}
export interface RequestMethod {
    (...args: any[]): any;
    serviceName?: string;
    methodName?: string;
}
export interface ApiBusResponse {
    items?: any[];
    nextToken?: number | string;
    [key: string]: any;
}
export interface ChannelState {
    fetching: boolean;
    lastFetched?: number;
    response?: ApiBusResponse;
    requestParams?: RequestParams;
    args?: ChannelArgs;
    error?: ApiError;
    preserve?: boolean;
    unconfirmed?: boolean;
    /**
     * If the channel has been invalidated, see `invalidateOn` for more details.
     */
    invalidated?: boolean;
    /**
     * See ChannelArgs.invalidateOn for more details
     */
    invalidateOn?: {
        methodName?: string;
        serviceName?: string;
    }[];
    /**
     * "signature" of the API used for this call, used for `invalidateOn` checking
     */
    requestMethod?: {
        methodName?: string;
        serviceName?: string;
    };
}
export interface ApiBusState {
    [key: string]: ChannelState | undefined;
}
export interface SubmitRequestAction<T = RequestParams> {
    type: typeof types['SUBMIT_REQUEST'];
    requestMethod: RequestMethod;
    requestParams: T;
    channel: string;
    args: ChannelArgs;
}
export interface RequestSuccessAction<T = ApiBusResponse> {
    type: typeof types['REQUEST_SUCCESS'];
    channel: string;
    response: T;
    lastFetched: number;
    args: ChannelArgs;
}
export interface RequestFailureAction {
    type: typeof types['REQUEST_FAILURE'];
    channel: string;
    error: ApiError;
    args: ChannelArgs;
}
export interface ChannelAction {
    type: string;
    channel: string;
}
/**
 * Maps message configuration value provided by the user to a proper message configuration object.
 * @param {Function:Promise} requestMethod The request method for the API call.
 * @param {string|array|object} userMessageConfig User provided message configuration value. Can be
 * a string, array of strings, or a message config object.
 *
 * @return Returns a message configuration object in this format:
 * { confirmKey, confirmParams, notifyKey, notifyParams }
 */
export declare const getMessageConfig: (requestMethod: RequestMethod, userMessageConfig: UserProvidedMessageConfig) => ChannelMessageConfig;
export declare const actions: {
    /**
     * @param {Function:Promise} requestMethod An async function that returns cdp-js/ApiResponse when await finishes.
     * @param {object} requestParams Any parameters to send in the body of the request.
     * @param {string} channel Optional. This defaults to the requestMethod methodName. It specifies the bucket to put the most recent invokation of the request.
     */
    submitRequest: (requestMethod: RequestMethod, requestParams: RequestParams, channel?: string, args?: UserProvidedChannelArgs) => SubmitRequestAction;
    /**
     * Creates a SUBMIT_REQUEST action with listAll argument set to true by default.
     * If listAll is set to true for a listing API, apiBusSaga will fetch all pages.
     */
    submitListRequest: (requestMethod: RequestMethod, requestParams: RequestParams, channel?: string, args?: UserProvidedChannelArgs) => SubmitRequestAction<RequestParams>;
    /**
     * @param {Function:Promise} requestMethod An async function that returns cdp-js/ApiResponse when await finishes.
     * @param {object} requestParams Any parameters to send in the body of the request.
     * @param {string} channel Required. This defaults to the requestMethod methodName. It specifies the bucket to put the most recent invokation of the request.
     * @param {string|array|object} messageConfig Required. Configuration for the confirm and notify
     * messages. Accepts a single paremeter string, and array of parameters, or the full configuration
     * object with keys and params.
     */
    submitConfirmedRequest: (requestMethod: RequestMethod, requestParams: RequestParams, channel: string, messageConfig?: UserProvidedMessageConfig, args?: UserProvidedChannelArgs) => SubmitRequestAction<RequestParams>;
    requestSuccess: (submitAction: SubmitRequestAction, response: ApiBusResponse, lastFetched: number) => RequestSuccessAction;
    requestFailure: (submitAction: SubmitRequestAction, error: ApiError) => RequestFailureAction;
    /**
     * Sets fetching to true for given channel. Can be used when a requets depends on another request
     * to finish but you want the UI to think the request is in progress.
     * @param {string|function} channel. Name of the channel or the API method
     */
    prepareChannel: (channel: RequestMethod | string) => ChannelAction;
    /**
     * Flushes the state.
     * Can be used to flush a specific api request state.
     * @param {string} Optional channel. If no channel is provided, all state is flushed.
     */
    flush: (channel?: string) => AnyAction;
    /**
     * Marks the request as confirmed, used by confirmedRequests.
     * Indicates that the user confirmed the request, and we have entered the fetching state.
     * @param channel
     */
    isConfirmed: (channel?: string) => AnyAction;
};
export default function apiBusReducer(state: ApiBusState | undefined, action: AnyAction): ApiBusState;
export declare const DEFAULT_CHANNEL_STATE: ChannelState;
export declare function requestState(state: RootState, channel: string): ChannelState;
/**
 * Returns true if any of the channels provided is still in fetching state.
 *
 * @param {object} state Store state
 * @param {array|string} channels List of channels to check
 */
export declare function getFetching(state: RootState, channels: Channels): boolean;
/**
 *
 * @param state store state
 * @param channels  List of channels to check
 * @returns true if any of the channels provided are in unconfirmed state
 */
export declare function getIsUnconfirmed(state: RootState, channels: Channels): boolean;
/**
 * Returns the error object from the first failed channel.
 *
 * @param {object} state Store state
 * @param {array|string} channels List of channels to check
 */
export declare function getError(state: RootState, channels: Channels): ApiError | undefined;
/**
 * Returns true if a fetch has been attempted or finished (regardless of outcome) on a
 * channel.
 *
 * @param {object} state Store state
 * @param {array|string} channels List of channels to check
 */
export declare function getFetchHasBeenAttempted(state: RootState, channels: Channels): boolean;
/**
 *
 * @param {object} state Store state
 * @param {string} channel Channel to get items from
 * @param {function} mapMethod Used for transforming items. If provided, items will be mapped to a
 * new array using this method.
 * T: Return type for the items in the array
 * Y: The intermediate type used if mapMethod is provided. The first argument to the mapMethod
 * will have type Y.
 */
export declare const getItems: <T = any, Y = T>(state: RootState, channel: string, mapMethod?: ((item: Y, ...args: any[]) => T) | undefined) => T[];
/**
 *
 * @param {object} state Store state
 * @param {string} channel Channel to get items from
 * @param {function} mapMethod Used for transforming items. If provided, items will be mapped to a
 * new array using this method.
 * T: Return type for the items in the array
 * Y: The intermediate type used if mapMethod is provided. The first argument to the mapMethod
 * will have type Y.
 */
export declare const getItem: <T = any, Y = T>(state: RootState, channel: string, itemPath?: string, mapMethod?: ((item: Y) => T) | undefined) => T | undefined;
export declare const getLastFetched: (state: RootState, channel: string) => number | undefined;
export declare const getRequestParams: (state: RootState, channel: string) => RequestParams | undefined;
export declare const getNextToken: (state: RootState, channel: string) => string | number | undefined;
export declare const FetchStatus: {
    FETCHING: string;
    REFRESHING: string;
    READY: string;
};
/**
 * Returns a fetch status that is calculated by looking at given channels and trying to
 * differentiate between the initial fetch and a subsequent fetch for refreshing data.
 * If all channels have a lastFetched value and none of them are refreshing this method returns READY.
 * If at least one channel does not have a lastFetched value, we return FETCHING.
 * If all channels have a lastFetched value and one of them is in fetching state we return REFRESHING.
 *
 * @param {object} state Store state
 * @param {array|string} channels Channels that store required data for our component
 */
export declare const getFetchStatus: (state: RootState, channels: Channels) => string;
export declare enum Status {
    UNSTARTED = "UNSTARTED",
    FETCHING_INITIAL = "FETCHING_INITIAL",
    FETCHING_FOLLOWING = "FETCHING_FOLLOWING",
    DONE = "DONE"
}
export declare const selectors: {
    getItem: <T = any, Y = T>(state: RootState, channel: string, itemPath?: string, mapMethod?: ((item: Y) => T) | undefined) => T | undefined;
    getItems: <T_1 = any, Y_1 = T_1>(state: RootState, channel: string, mapMethod?: ((item: Y_1, ...args: any[]) => T_1) | undefined) => T_1[];
    getError: typeof getError;
    getRequestState: typeof requestState;
    getStatus: (state: RootState, channels: Channels) => Status;
    isUnstarted: (state: RootState, channels: Channels) => boolean;
    isFetchingOrUnstarted: (state: RootState, channels: Channels) => boolean;
    isFetchingOrCompleted: (state: RootState, channels: Channels) => boolean;
    isFetching: typeof getFetching;
    hasNextToken: (state: RootState, channel: string) => boolean;
    getLastFetched: (state: RootState, channel: string) => number | undefined;
    getIsInvalidated: (state: RootState, channel: string) => boolean | undefined;
};
export {};
