// Copyright (c) 2018 Cloudera, Inc. All rights reserved.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import pickBy from 'lodash/pickBy';
import get from 'lodash/get';
import pick from 'lodash/pick';
export var types = {
    NAVIGATE: 'router/NAVIGATE',
    NAVIGATE_SUCCESS: 'router/NAVIGATE_SUCCESS',
    UPDATE_SEARCH_PARAMS: 'router/UPDATE_SEARCH_PARAMS',
    SET_SEARCH_PARAMS: 'router/SET_SEARCH_PARAMS'
};
export var actions = {
    navigate: function (id, params, searchParams, opts, path) {
        if (params === void 0) { params = {}; }
        if (searchParams === void 0) { searchParams = {}; }
        if (opts === void 0) { opts = {}; }
        return ({
            type: types.NAVIGATE,
            id: id,
            params: params,
            searchParams: searchParams,
            opts: opts,
            path: path
        });
    },
    navigateSuccess: function () { return ({ type: types.NAVIGATE_SUCCESS }); },
    updateSearchParams: function (searchParams) { return ({
        type: types.UPDATE_SEARCH_PARAMS,
        searchParams: searchParams,
        opts: { replace: false }
    }); },
    setSearchParams: function (searchParams) { return ({
        type: types.SET_SEARCH_PARAMS,
        searchParams: searchParams,
        opts: { replace: true }
    }); }
};
export var initialState = {
    // Ready flag controls the component rendering that is triggered by route changes. If set to false
    // route driven components will not be re-rendered until ready becomes true.
    ready: true,
    id: undefined,
    params: {},
    searchParams: {},
    opts: {},
    route: null,
    path: ''
};
export default function getRouterReducer(routesMap) {
    return function (state, action) {
        if (state === void 0) { state = initialState; }
        switch (action.type) {
            case types.NAVIGATE:
                var navigateAction = action;
                return {
                    id: navigateAction.id,
                    ready: false,
                    params: navigateAction.params,
                    searchParams: navigateAction.searchParams,
                    opts: navigateAction.opts || {},
                    route: routesMap[navigateAction.id],
                    path: navigateAction.path
                };
            case types.NAVIGATE_SUCCESS:
                return __assign(__assign({}, state), { ready: true });
            case types.SET_SEARCH_PARAMS:
                var setSsearchParamAction = action;
                return __assign(__assign({}, state), { searchParams: __assign({}, setSsearchParamAction.searchParams) });
            case types.UPDATE_SEARCH_PARAMS:
                var updateSearchParamAction = action;
                return __assign(__assign({}, state), { 
                    // merge old with new and remove params with undefined values
                    searchParams: pickBy(__assign(__assign({}, state.searchParams), updateSearchParamAction.searchParams), function (v) { return v !== undefined; }), opts: updateSearchParamAction.opts });
            default:
                return state;
        }
    };
}
// Selectors
export var getRoutingId = function (state) { return state.routing.id; };
export var getRoutingPath = function (state) { return state.routing.path; };
/**
 *  * Returns the router parameter value for the given parameter name.
 */
export var getRoutingParam = function (state, paramName, defaultValue) {
    return state.routing.params[paramName] || defaultValue;
};
/**
 * Returns the searchParams object from the route state. If paramNames is provided, only the keys
 * listed in the array are returned.
 */
export var getSearchParams = function (state, paramNames) {
    if (paramNames === void 0) { paramNames = []; }
    var searchParams = get(state, "routing.searchParams", {});
    if (!paramNames || paramNames.length === 0) {
        return searchParams;
    }
    return pick(searchParams, paramNames);
};
/**
 * Returns the search parameter value for the given parameter name.
 * Use a bit of magic TS to make it return `string` type if defaultValue
 * is specified as opposed to `string | undefined`.
 */
export var getSearchParam = function (state, paramName, defaultValue) { return state.routing.searchParams[paramName] || defaultValue; };
/**
 * Narrow SearchParam value by removing `string[]` which is rarely used.
 * If used, then pick the first value in the array.
 */
export var convertToSimpleValue = function (value) {
    return Array.isArray(value) ? value[0] : value;
};
/**
 * Returns the search parameter value for the given parameter name. If search param is an
 * array, will return the first value.
 */
export var getSimpleSearchParam = function (state, paramName, defaultValue) {
    var value = getSearchParam(state, paramName, defaultValue);
    return Array.isArray(value) ? value[0] : value;
};
/**
 * Returns the search parameter value for the given parameter name in number format. Default value
 * is returned if the value is not a number.
 */
export var getNumberSearchParam = function (state, paramName, defaultValue) {
    if (defaultValue === void 0) { defaultValue = undefined; }
    var strValue = getSearchParam(state, paramName, defaultValue);
    if (!strValue || Array.isArray(strValue)) {
        return defaultValue;
    }
    var numberValue = parseInt(strValue, 10);
    return !isNaN(numberValue) ? numberValue : defaultValue;
};
/**
 * Given a current path and a new path, resolve the path.
 * - if the new path is absolute, return that
 * - if the new path is relative, combine the paths and if necessary travel upwards
 *
 * Should follow nodeJS path.resolve method
 *
 */
export var resolvePath = function (base, newPath) {
    var isAbsolute = newPath.startsWith('/');
    if (isAbsolute) {
        return newPath;
    }
    if (!base.startsWith('/')) {
        // nodejs path.resolve would rely on the cwd in this case.
        // This is not an abstraction we need, so we'll limit us to absolute base path.
        throw new Error("Only absolute paths are supported for 'base'. Got '".concat(base, "'"));
    }
    var isRelativeBackwards = newPath.startsWith('../') || newPath === '..';
    var normalizedBase = base.replace(/\/+$/, '');
    var normalizedNewPath = newPath.replace(/^\.\//, '').replace(/\/+$/, '');
    if (!isRelativeBackwards) {
        var path = "".concat(normalizedBase, "/").concat(normalizedNewPath);
        return path;
    }
    var newPathParts = normalizedNewPath.split('/');
    var baseParts = normalizedBase.split('/');
    while (newPathParts[0] === '..') {
        newPathParts.shift();
        baseParts.pop();
    }
    if (newPathParts.length) {
        return "".concat(baseParts.join('/'), "/").concat(newPathParts.join('/'));
    }
    return baseParts.join('/') || '/';
};
