import type { ItemType } from 'antd/es/menu/hooks/useItems';
export interface SimpleMap<T = any> {
    [key: string]: T;
}
export interface Error {
    [key: string]: any;
}
export interface ApiError {
    code: string;
    title: string;
    message: string;
    requestId?: string;
}
export interface StringMap {
    [key: string]: string | undefined;
}
/**
 * Converts a type for a static object to StaticMap type which allows lookups for keys that don't
 * exist and type them as undefined.
 * E.g. const X = {foo:'123'}; // X.bar is an invalid lookup since type of X is {foo:string}
 * S, this helper type returns a more flexible type definition that says typeof X.bar is undefined.
 */
type StaticMap<T extends object> = {
    readonly [key in keyof T]: T[keyof T];
} & {
    readonly [key: string]: undefined;
} & {
    readonly [key: number]: undefined;
};
/**
 * Helper that converts a given object to StaticMap.
 * @param map Object to convert to a StaticMap
 */
export declare function makeStaticMap<T extends object>(map: T): StaticMap<T>;
/**
 * Omit for union types since regular Omit helper does not work properly on union types.  E.g.:
 * type A = {type: string; name: string}
 * type B = {type: number; name: string}
 * type C = A | B;
 *
 * Omit<C, 'type'> does not return what you'd expect but OmitFromUnion<C, 'type'> works as expected.
 */
export type OmitFromUnion<T, K extends keyof any> = T extends any ? Omit<T, K> : never;
export * from "cdp-js/dist/lib/utilityTypes";
/**
 * Type for antd menu items which has props and typing requirement of
 * official antd ItemType but allows us to add data-event tracking possibilities.
 *
 * Antd expects us to define menu items as ItemType: https://ant.design/components/menu/#ItemType
 * However, this type doesn't allow us to add data-event related props
 * which are useful for event tracking in google analytics, that's why we added
 * this new type.
 */
export type CuixMenuItem = ItemType & {
    'data-event-category'?: string;
    'data-event'?: string;
    'data-event-label'?: string;
};
export declare enum Cloud {
    AmazonWebServices = "AWS",
    GoogleCloudPlatform = "GCP",
    MicrosoftAzure = "AZURE",
    RedHatOpenShift = "OPENSHIFT"
}
/**
 * Type defining allowable values for colspans in the CDL grid system.
 */
export type Colspan = 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12;
/**
 * Type defining CDL color ramps.
 */
export declare enum Color {
    Gray = "CDL_GRAY",
    Red = "CDL_RED",
    Orange = "CDL_ORANGE",
    Amber = "CDL_AMBER",
    Pear = "CDL_PEAR",
    Green = "CDL_GREEN",
    Spearmint = "CDL_SPEARMINT",
    Blue = "CDL_BLUE",
    Indigo = "CDL_INDIGO",
    Purple = "CDL_PURPLE",
    Pink = "CDL_PINK"
}
/**
 * Type definind CDL shadow styles.
 */
export type Shadow = 'none' | 'small' | 'medium' | 'large';
/**
 * This is a hack so that we can use `theme.variant` functions we get from
 * `styled-theming` outside of `styled-components`. For example, let's say we
 * have this function to get a color variant:
 *
 * ```
 * const color: ThemeVariantFunction = theme.variants('mode', 'color', {
 *   [Color.Red]: { default: cdlRed900, specialdark: cdlRed900 },
 *   [Color.Green]: { default: cdlGreen900, specialdark: cdlGreen900 },
 *   [Color.Blue]: { default: cdlBlue900, specialdark: cdlBlue900 }
 * });
 * ```
 *
 * The `styled-theming` library passes the full props object implicitly when
 * this is called in the context of a styled component, but if in some comopnent
 * we want to call it arbitrarily, like this:
 *
 * ```
 * const variantColor = color({ theme: { mode: 'default' }, color: Color.Red })
 * ```
 *
 * we will get a type error due to the `theme` prop we're trying to include,
 * unless we declare it as a `ThemeVariantFunction`.
 */
type PropsWithTheme = {
    theme: {
        mode: string;
    };
} & any;
export type ThemeVariantFunction = (props: PropsWithTheme) => string;
