// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
/**
 * From several domains, find any common domain e.g.
 *
 * - test.cloudera.com, help.cloudera.com, cloudera.com
 *      => common domain: cloudera.com
 * - example.com, cloudera.com, test.cloudera.com
 *      => common domain: false
 *
 * Expects an array of items, and and accessor to get the domain from the item.
 * This is done so we can iterate over existing complex objects.
 *
 * Example:
 *
 * ```js
 * const list = [{ name: 'test', domain: 'x.y.z', ...}, { name: 'other, domain: 'y.y.z', ...}, ...];
 * const accessor = (listItem) => listItem.domain;
 * const common = findCommonDomain(list, accessor);
 * ```
 *
 * If you have an plain domain array, you can pass in identify fn to accessor ( item => item ).
 */
export var findCommonDomain = function (list, accessor) {
    if (!list.length) {
        return false;
    }
    // split the first url up e.g. arne.my.domain.com => arne, my, domain, com
    var parts = accessor(list[0]).split('.');
    // create potential common parts arne.my.domain.com => .my.domain.com, .domain.com, .com
    var potentialCommonParts = parts.slice(1).map(function (_part, index) {
        return '.' + parts.slice(index + 1).join('.');
    });
    // find the most specific domain that is used in all items
    var commonPart = potentialCommonParts.find(function (commonPart) {
        return list.every(function (item) { return accessor(item).endsWith(commonPart); });
    });
    return commonPart || false;
};
