// Licensed to Cloudera, Inc. under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  Cloudera, Inc. licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
/**
 * @module utils/humanize
 * @description This module provides humanization related functionality.
 */
// Unit suffixes for values representing only magnitude (pop pop).
var COUNT_UNITS = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'];
// Unit suffixes for values representing bytes.
var BYTE_UNITS = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB' /* 2 ^ 60 */];
// Unit suffixes for values representing bytes per second.
var BYTE_PER_SECOND_UNITS = ['B/s', 'KiB/s', 'MiB/s', 'GiB/s', 'TiB/s', 'PiB/s', 'EiB/s'];
/**
 * Returns string representation of a given number with trailing zeros removed.
 */
var stringifyNumber = function (y, digits) {
    if (typeof y !== 'number' || !isFinite(y)) {
        return '';
    }
    var str;
    var precisionLimit = 1 / (2 * Math.pow(10, digits));
    var absY = Math.abs(y);
    if (digits > 0 && absY > 0 && absY < precisionLimit) {
        // y is too small so Number.toFixed will just return 0. Use Number.toPrecision
        // in order to preserve the value.
        // (i.e., for digits == 2, y is between -0.005 and 0.005 excluding 0)
        str = y.toPrecision(digits);
    }
    else {
        str = y.toFixed(digits);
    }
    // Remove trailing zeros by converting to numbe and back to string.
    return Number(str).toString();
};
var createUnitHumanizer = function (units, base, useSpace) {
    return function humanizeIt(rawValue) {
        // Coerce to a number.
        var value = Number(rawValue);
        if (value < 0) {
            return '-' + humanizeIt(-1 * value);
        }
        var spacer = useSpace ? ' ' : '';
        var index = 0, resultNum;
        if (value < base) {
            if (value < 1) {
                resultNum = stringifyNumber(value, 1);
            }
            else {
                resultNum = stringifyNumber(value, 2);
            }
        }
        else {
            index = Math.floor(Math.log(value) / Math.log(base));
            index = Math.min(units.length - 1, index);
            resultNum = stringifyNumber(value / Math.pow(base, index), 1);
        }
        return resultNum + spacer + units[index];
    };
};
/**
 * Humanize values to two significant digits, with the exception of 3 digit numbers without decimals.
 * e.g.
 * 0.00123 => 0.0012
 * 0.0123 => 0.012
 * 0.123 => 0.12
 * 1.23 => 1.2
 * 12.3 => 12
 * 123 => 123 (3 digits of significant digits)
 * 1230 => 1.2k
 * 12300 => 12k
 * 123000 => 123k (3 digits of significant digits)
 *
 * This was motivated by inconsistentency in the `humanizeCount` for certain transition zones for small
 * values e.g
 * 0.006 -> 0.006 <--- kept
 * 0.04 -> 0.04
 * 0.06 -> 0.1  <--- unexpected rounding
 * 0.14 -> 0.1
 * which can for example cause the same number to appear twice on a graph axis.
 *
 * `createUnitHumanizerV2` is also more consitent for larger numbers e.g.
 * createUnitHumanizerV2: 340200 => 340k (stick to at most 3 significant digits)
 * humanizeCount: 340200 -> 340.2k (allows 4 signifcant digits in this case vs 1 in others)
 *
 * Going forward it might be worthwhile to switch most existing humanizeCount usage over to createUnitHumanizerV2, but
 * that is a bigger task.
 */
var createUnitHumanizerV2 = function (units, base, useSpace, signicantDigits) {
    if (signicantDigits === void 0) { signicantDigits = 2; }
    var humanizeFn = function (rawValue) {
        var value = Number(rawValue);
        if (value < 0) {
            return '-' + humanizeFn(-1 * value);
        }
        if (value === 0) {
            return "0";
        }
        var unitIndex = 0;
        unitIndex = Math.floor(Math.log(value) / Math.log(base));
        unitIndex = Math.min(units.length - 1, unitIndex);
        unitIndex = Math.max(unitIndex, 0);
        value = value / Math.pow(base, unitIndex);
        if (value < base) {
            if (value < 100 || signicantDigits > 2) {
                var factor = Math.floor(Math.log10(value)) - signicantDigits + 1;
                value = value * Math.pow(10, -factor);
                value = Math.round(value) / Math.pow(10, -factor);
            }
            else {
                // for values between 100 and 999, we have to display 3 digits, so
                // make tham all count by just removing any decimals.
                value = Math.round(value);
            }
        }
        return "".concat(value).concat(useSpace ? ' ' : '').concat(units[unitIndex]);
    };
    return humanizeFn;
};
var humanize = {
    // The intent of this function is to render the count using only four
    // characters, if possible. Knowing the max length of the output of this
    // function makes it easier to layout the UI in certain cases,
    // e.g. histograms.
    humanizeCount: createUnitHumanizer(COUNT_UNITS, 1000, false),
    humanizeCountV2: createUnitHumanizerV2(COUNT_UNITS, 1000, false),
    humanizeBytes: createUnitHumanizer(BYTE_UNITS, 1024, true),
    humanizeBytesV2: createUnitHumanizerV2(BYTE_UNITS, 1024, true),
    humanizeBytesPerSecond: createUnitHumanizer(BYTE_PER_SECOND_UNITS, 1024, true),
    stringifyNumber: stringifyNumber,
    humanizeByteSeconds: function (value) {
        return "".concat(humanize.humanizeBytesV2(value), "/s");
    }
};
export default humanize;
