// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
import moment from 'moment';
var MS_IN_S = 1000;
var MS_IN_MIN = MS_IN_S * 60;
var MS_IN_HOUR = MS_IN_MIN * 60;
var MS_IN_DAY = MS_IN_HOUR * 24;
var MS_IN_WEEK = MS_IN_DAY * 7;
export var periods = {
    MIN_10: { value: 10 * MS_IN_MIN, startOf: 'hour' },
    MIN_20: { value: 20 * MS_IN_MIN, startOf: 'hour' },
    MIN_30: { value: 30 * MS_IN_MIN, startOf: 'hour' },
    HOUR_1: { value: 1 * MS_IN_HOUR, startOf: 'hour' },
    HOUR_2: { value: 2 * MS_IN_HOUR, startOf: 'day' },
    HOUR_4: { value: 4 * MS_IN_HOUR, startOf: 'day' },
    HOUR_6: { value: 6 * MS_IN_HOUR, startOf: 'day' },
    HOUR_12: { value: 12 * MS_IN_HOUR, startOf: 'day' },
    DAY_1: { value: 1 * MS_IN_DAY, startOf: 'day' },
    // not for multiple, min/hour we align it to a bigger unit e.g. hour/day, but for days/weeks we just align
    // it to the same unit, as 7 days isn't easily divisible, and 4 weeks is different than a month.
    DAY_2: { value: 2 * MS_IN_DAY, startOf: 'day' },
    DAY_4: { value: 4 * MS_IN_DAY, startOf: 'day' },
    WEEK_1: { value: 1 * MS_IN_WEEK, startOf: 'week' },
    WEEK_2: { value: 2 * MS_IN_WEEK, startOf: 'week' },
    WEEK_4: { value: 4 * MS_IN_WEEK, startOf: 'week' }
};
var periodList = [
    periods.MIN_10,
    periods.MIN_20,
    periods.MIN_30,
    periods.HOUR_1,
    periods.HOUR_2,
    periods.HOUR_4,
    periods.HOUR_6,
    periods.HOUR_12,
    periods.DAY_1,
    periods.DAY_2,
    periods.DAY_4,
    periods.WEEK_1,
    periods.WEEK_2,
    periods.WEEK_4
];
/**
 * Given a time range, generate time "buckets" that aligns nicely with time.
 * This is useful if you want to present the data to the user in time buckets, as it will present much
 * nice e.g. with a bucket from 5-6PM rather than a bucket from 5:12.124PM to 6:23.123PM.
 * It will return as many buckets we can nicely divide up into that is less than maxBuckets.
 * e.g. if you passed in 3:12am as start time and  4:30pm as end time and max 15 buckets, it would
 * probably give you hourly buckets starting at 3:00am and ending at 5:00pm
 */
export var getTimeBucketsConfigFor = function (_a) {
    var startTime = _a.startTime, endTime = _a.endTime, maxBuckets = _a.maxBuckets, _b = _a.minBucketDuration, minBucketDuration = _b === void 0 ? 0 : _b;
    var timeSpan = endTime - startTime;
    // find first period that gives us less than maxBuckets
    var period = periodList.find(function (unit) {
        return timeSpan / unit.value <= maxBuckets && unit.value >= minBucketDuration;
    }) || periodList[periodList.length - 1];
    // we'll "anchor" our buckets to a nice round date (e.g. day/hour change) so we'll hav buckets
    // e.g. from 8:00pm to 9:00pm instead of 8.12pm to 9.12pm
    var anchorTime = moment(startTime).startOf(period.startOf);
    var anchorToStartTime = startTime - anchorTime.valueOf();
    var periodsUntilStart = anchorToStartTime / period.value;
    var bucketsStartTime = Math.floor(periodsUntilStart) * period.value + anchorTime.valueOf();
    var numBuckets = Math.ceil((endTime - bucketsStartTime) / period.value);
    var bucketsEndTime = numBuckets * period.value + bucketsStartTime;
    return {
        /**
         * Our bucket start time, this will always be equal or less than the startTime given , but alway aligned
         * to a "nice" time.
         */
        bucketsStartTime: bucketsStartTime,
        /**
         * Our bucket end time, this will be equal or more than our endTime, but aligned to a "nice" time.
         */
        bucketsEndTime: bucketsEndTime,
        /**
         * Exact number of buckets it takes to fill from `bucketStartTime` to `bucketsEndTime`
         */
        numBuckets: numBuckets,
        /**
         * Duration of a single bucket, this should be one of one the "nice" values specifid in `periods`
         */
        bucketDuration: period.value
    };
};
/**
 * Creates an array of buckets given a bucket config, each with start/endTime
 */
export var getTimeBucketArrayFromConfig = function (_a) {
    var bucketsStartTime = _a.bucketsStartTime, numBuckets = _a.numBuckets, bucketDuration = _a.bucketDuration;
    var buckets = [];
    for (var index = 0; index < numBuckets; index++) {
        var startTime = bucketsStartTime + index * bucketDuration;
        buckets.push({
            startTime: startTime,
            endTime: startTime + bucketDuration
        });
    }
    return buckets;
};
