// Licensed to Cloudera, Inc. under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  Cloudera, Inc. licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
import moment from 'moment-timezone';
import humanize from './humanize';
import i18n from './i18n';
var SHORT_TIME_UNITS = ['d', 'h', 'm', 's', 'ms'];
var LONG_TIME_UNITS = ['day', 'hour', 'mimute', 'second', 'millisecond'];
var DENOMS = [86400000, 3600000, 60000, 1000, 1];
/**
 * Allows timeUtil to offset any time to take into account server time delta.
 */
var timeDelta = 0;
/**
 * @description Utilities for displaying human readable time.
 */
var timeUtil = {
    setDelta: function (delta) { return (timeDelta = delta); },
    // only exposed for testing.
    _currentTimezone: function () { return moment.tz.guess(); },
    /**
     * Returns the abbrevation of current time zone (E.g., PDT)
     */
    currentTimezoneShortName: function () { return moment().tz(timeUtil._currentTimezone()).format('z'); },
    /**
     * Returns a moment object for the given date in user's timezone.
     * @param  {string|number|Date} date Input date. Any input that is accepted by moment.js is valid.
     * @return {Moment}
     */
    localDate: function (date) { return moment.tz(date, timeUtil._currentTimezone()).subtract(timeDelta, 'ms'); },
    /**
     * Returns a formatted moment date.
     * @param  {string|number|Date} date Input date. Any input that is accepted by moment.js is valid.
     * @param  {string} format The moment format.
     * @return {string}
     */
    formattedDate: function (date, format) {
        if (!date) {
            return '';
        }
        var momentDate = this.localDate(date);
        if (!momentDate || !momentDate.isValid()) {
            return '';
        }
        if (momentDate.milliseconds() === 0) {
            format = format.replace(/\.SSS/, '');
        }
        return momentDate.format(format);
    },
    /**
     * Returns a short date time string like "08/07/2017 1:19 PM"
     * @return {string}
     */
    shortDateTime: function (date) { return timeUtil.formattedDate(date, 'L LT'); },
    /**
     * Returns a time string like "1:19 PM"
     * @return {string}
     */
    shortTime: function (date) { return timeUtil.formattedDate(date, 'LT'); },
    shortTimeAndMs: function (date) { return timeUtil.formattedDate(date, 'h:mm:ss.SSS A'); },
    /**
     * Returns a time string like "1:19 PM"
     * @return {string}
     */
    shortDate: function (date) { return timeUtil.formattedDate(date, 'L'); },
    /**
     * Returns a medium date time string like "08/07/2017 1:19 PM PDT"
     * @return {string}
     */
    mediumDateTime: function (date) { return timeUtil.formattedDate(date, 'L LT z'); },
    /**
     * Returns a medium time string like "1:19 PM PDT"
     * @return {string}
     */
    mediumTime: function (date) { return timeUtil.formattedDate(date, 'LT z'); },
    /**
     * Returns a long date string like "January 30, 2018"
     * @return {string}
     */
    longDate: function (date) { return timeUtil.formattedDate(date, 'LL'); },
    /**
     * Returns a long date string like "January 30, 2018 11:20 AM PDT"
     * @return {string}
     */
    longDateTime: function (date) { return timeUtil.formattedDate(date, 'LLLL z'); },
    /** @return {string} */
    longTime: function (date) { return timeUtil.formattedDate(date, 'H:mm:ss.SSS ZZ'); },
    /**
     * Returns a date string of the form "1970-01-01 12:00:00 AM UTC".
     * @return {string}
     */
    cdp_12HDateTime: function (date) { return timeUtil.formattedDate(date, 'YYYY-MM-DD hh:mm:ss A z'); },
    /** @return {string} */
    dateMonth: function (date) { return timeUtil.formattedDate(date, 'MMM'); },
    /** @return {string} */
    dateYear: function (date) { return timeUtil.formattedDate(date, 'YYYY'); },
    /**
     * Parses a ms value into days, hours, minutes, etc.
     */
    parseMs: function (ms) {
        if (ms === void 0) { ms = 0; }
        var parsed = [];
        // we use denoms to split up the ms into chunks.  e.g. seconds, minutes, hours
        DENOMS.forEach(function (denom) {
            parsed.push(Math.floor(ms / denom));
            ms = ms % denom;
        });
        return parsed;
    },
    /**
     * Rounds parts of a parsed duration object
     */
    roundParsedMs: function (parts, units) {
        if (units === void 0) { units = SHORT_TIME_UNITS; }
        return parts.map(function (part, i, all) {
            var next = all[i + 1] || 0;
            var ratio = DENOMS[i] / (DENOMS[i + 1] || 1);
            return {
                floor: part,
                rounded: Math.round(part + next / ratio),
                unit: units[i]
            };
        });
    },
    /**
     * Formats a duration in ms (E.g., 10h 22m)
     * @param  {number} ms       Duration in ms
     * @param  {number} parts    Number of parts we want in the formatted string
     * @param  {boolean} ignoreMs Whether to ignore milliseconds
     */
    formatMs: function (ms, parts, ignoreMs, long) {
        if (parts === void 0) { parts = 2; }
        if (ignoreMs === void 0) { ignoreMs = false; }
        if (long === void 0) { long = false; }
        if (ms < 1) {
            return ignoreMs ? '0s' : '0ms';
        }
        if (ignoreMs) {
            if (ms < 1000) {
                return '< 1s';
            }
            ms = Math.round(ms / 1000) * 1000;
        }
        var parsed = timeUtil.roundParsedMs(timeUtil.parseMs(ms), long ? LONG_TIME_UNITS : SHORT_TIME_UNITS);
        var values = parsed.reduce(function (memo, part) { return (memo.length || part.floor ? memo.concat(part) : memo); }, []); // Remove leading zeroes
        var shown = values.slice(0, parts);
        var last = shown.pop();
        var formatted = shown.map(function (part) { return part.floor + part.unit; });
        if (last && last.rounded) {
            formatted.push(last.rounded + last.unit);
        }
        return formatted.join(' ');
    },
    formatSeconds: function (seconds) {
        return timeUtil.formatMs(seconds * 1000);
    },
    // Convert nanoseconds to a human-friendly string.
    // It also handles microseconds. Past that, it defers
    // to humanizeMilliseconds.
    formatNanoseconds: function (nanos) {
        // Coerce to a number.
        nanos = Number(nanos);
        if (nanos < 1000) {
            return humanize.stringifyNumber(nanos, 0) + 'ns';
        }
        var micros = nanos * Math.pow(10, -3);
        if (micros < 1000) {
            return humanize.stringifyNumber(micros, 2) + '\u00B5s';
        }
        return timeUtil.formatMs(nanos * Math.pow(10, -6));
    },
    /**
     * Calculates the number of days left until a user's workload password expires.
     * In case it already expired, it returns a negative number.
     */
    getDaysUntil: function (expirationDate) {
        if (!expirationDate || !moment(expirationDate, moment.ISO_8601).isValid()) {
            return;
        }
        var now = new Date();
        var expirationTime = new Date(expirationDate);
        var msInDay = 1000 * 3600 * 24;
        var daysUntilExpiration = Math.floor((expirationTime.getTime() - now.getTime()) / msInDay);
        return daysUntilExpiration;
    },
    /**
     * In some cases we would like to display how many days are left
     * until something expires (e.g. a password or a trial period).
     * This function returns how many days are left in the format of
     * - 1 day
     * - 4 days
     * - less than a day
     */
    getTextOfDaysUntil: function (daysLeftUntil) {
        if (daysLeftUntil < 0) {
            return undefined;
        }
        var dayText = daysLeftUntil > 1 ? i18n.t('label.users.days') : i18n.t('label.users.day');
        var dayCount = daysLeftUntil >= 1 ? daysLeftUntil : i18n.t('label.users.lessThanA');
        return "".concat(dayCount, " ").concat(dayText);
    }
};
export default timeUtil;
