// Copyright (c) 2021 Cloudera, Inc. All rights reserved.
/**
 * Calculates if a user's workload password change is enabled.
 * Workload passwords have a passwordMinLifetimeDate property which is the date until a workload password can not yet be changed.
 * If current date is before this date, changing workload passwords is disabled.
 */
export var passedMinLifetimeDate = function (passwordMinLifetimeDate) {
    var now = new Date();
    // password can be changed if either it doesn't have a minLifetimeDate or it has already passed
    return passwordMinLifetimeDate ? passwordMinLifetimeDate <= now.toISOString() : true;
};
/**
 * Helper function to determine if we need to show notification about upcoming expiration of workload password.
 *
 * We show notification only when password is set, current date is beyond passwordMinLifetimeDate (if it is defined)
 * and workload password will expire in the next 10 days.
 *
 * @param {WorkloadPasswordDetails} workloadPasswordDetails details about password expiration.
 * Returns if notification should be shown, if password already expired and how many days are left until password expires.
 *    If the password already expired, daysUntilExpiration can be a negative number.
 */
export var getExpiredWorkloadPasswordDetails = function (workloadPasswordDetails) {
    if (workloadPasswordDetails) {
        var isPasswordSet = workloadPasswordDetails.isPasswordSet, passwordMinLifetimeDate = workloadPasswordDetails.passwordMinLifetimeDate, passwordExpirationDate = workloadPasswordDetails.passwordExpirationDate;
        var passwordCanBeChanged = passedMinLifetimeDate(passwordMinLifetimeDate);
        var daysUntilExpiration = getDaysUntil(passwordExpirationDate);
        if (isPasswordSet &&
            passwordCanBeChanged &&
            passwordExpirationDate &&
            daysUntilExpiration < 10) {
            return {
                showNotification: true,
                daysUntilExpiration: daysUntilExpiration,
                expired: daysUntilExpiration < 0
            };
        }
    }
    return {
        showNotification: false
    };
};
/**
 * Calculates the number of days left until a date.
 * In case it already expired, it returns a negative number.
 */
var getDaysUntil = function (date) {
    if (!date) {
        return;
    }
    var now = new Date();
    var expirationTime = new Date(date);
    var msInDay = 1000 * 3600 * 24;
    var daysUntilDate = Math.floor((expirationTime.getTime() - now.getTime()) / msInDay);
    return daysUntilDate;
};
